/***************************************************************************
Name: CS Lite
Description: Control cookie permissions.
Author: Ron Beckman
Homepage: http://forum.softwareblaze.com

Copyright (C) 2007  Ron Beckman

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

Free Software Foundation, Inc.
51 Franklin Street
Fifth Floor
Boston, MA  02110-1301
USA
***************************************************************************/



var csMenuItemsObserver = {
	init: function() {
		var os = this.getObserver();
		os.addObserver(this, 'cookiesafe-menuitems-changed', false);
	},

	uninit: function() {
		var os = this.getObserver();
		os.removeObserver(this, 'cookiesafe-menuitems-changed');
	},

	QueryInterface : function(aIID) {
		if (aIID.equals(Components.interfaces.nsISupports) ||
		    aIID.equals(Components.interfaces.nsIObserver))
			return this;
		throw Components.results.NS_NOINTERFACE;
	},

	getObserver: function() {
		return Components.classes["@mozilla.org/observer-service;1"].
		getService(Components.interfaces.nsIObserverService);
	},

	getCS: function() {
		return Components.classes['@mozilla.org/CookieSafe;1'].
		createInstance(Components.interfaces.nsICookieSafe);
	},

	getCSHiddenMenuItems: function() {
		return Components.classes['@mozilla.org/CSHiddenMenuItems;1'].
		createInstance(Components.interfaces.nsICSHiddenMenuItems);
	},

	getCSLast10Hosts: function() {
		return Components.classes['@mozilla.org/CSLast10Hosts;1'].
		getService(Components.interfaces.nsICSLast10Hosts);
	},

	getCSTempExceptions: function() {
		return Components.classes['@mozilla.org/CSTempExceptions;1'].
		getService(Components.interfaces.nsICSTempExceptions);
	},

	observe: function(aSubject, aTopic, aData) {
		if (aTopic=='cookiesafe-menuitems-changed') {
			cookiesafeOptions.createListItems();
		}
	}
};

var cookiesafeOptions = {

	getObserver: function() {
		return Components.classes["@mozilla.org/observer-service;1"].
		getService(Components.interfaces.nsIObserverService);
	},

	getCS: function() {
		return Components.classes['@mozilla.org/CookieSafe;1'].
		createInstance(Components.interfaces.nsICookieSafe);
	},

	getCSHiddenMenuItems: function() {
		return Components.classes['@mozilla.org/CSHiddenMenuItems;1'].
		createInstance(Components.interfaces.nsICSHiddenMenuItems);
	},

	getCSLast10Hosts: function() {
		return Components.classes['@mozilla.org/CSLast10Hosts;1'].
		getService(Components.interfaces.nsICSLast10Hosts);
	},

	getCSTempExceptions: function() {
		return Components.classes['@mozilla.org/CSTempExceptions;1'].
		getService(Components.interfaces.nsICSTempExceptions);
	},

	getStr: function(name) {
		return document.getElementById("cookiesafe-bundle").getString(name);
	},

	getPrefs: function() {
		return Components.classes["@mozilla.org/preferences-service;1"].
		getService(Components.interfaces.nsIPrefService).
		getBranch("cookiesafe.");
	},

	getGlobalPrefs: function() {
		return Components.classes["@mozilla.org/preferences-service;1"].
		getService(Components.interfaces.nsIPrefService).
		getBranch("network.cookie.");
	},

	getWin: function(name) {
		return Components.classes["@mozilla.org/appshell/window-mediator;1"].
		getService(Components.interfaces.nsIWindowMediator).
		getMostRecentWindow(name);
	},

	onDialogLoad: function() {
		//register observer
		csMenuItemsObserver.init();

		var dom = this.getDOM();
		var prefs = this.getPrefs();
		var gPrefs = this.getGlobalPrefs();

		//set cookiesafe checkbox prefs
		for (var i=0; i<dom.chk.cs.length; ++i) {
			dom.chk.cs[i].checked = prefs.getBoolPref(dom.chk.cs[i].id);
		}

		//set global checkbox prefs
		for (var i=0; i<dom.chk.gbl.length; ++i) {
			dom.chk.gbl[i].checked = gPrefs.getBoolPref(dom.chk.gbl[i].id);
		}

		//set cookiesafe menulist prefs
		for (var i=0; i<dom.mnu.cs.length; ++i) {
			if (dom.mnu.cs[i].id=='defaultSkin') {
				dom.mnu.cs[i].selectedIndex = this.convertSkinUrl(prefs.getCharPref(dom.mnu.cs[i].id));
			} else {
				dom.mnu.cs[i].selectedIndex = prefs.getIntPref(dom.mnu.cs[i].id);
			}
		}

		//set global menulist prefs
		for (var i=0; i<dom.mnu.gbl.length; ++i) {
			dom.mnu.gbl[i].selectedIndex = gPrefs.getIntPref(dom.mnu.gbl[i].id);
		}

		//set global textbox prefs
		dom.txt.gbl[0].value = gPrefs.getIntPref(dom.txt.gbl[0].id);

		//show global textbox if lifetimePolicy is set to 3
		if (parseInt(dom.mnu.gbl[1].selectedIndex)==3) this.lifetimeDays(false);

		//create cookiesafe listbox items
		this.createListItems();
	},

	exit: function() {
		csMenuItemsObserver.uninit();
	},

	createListItems: function() {
		var listitem,listcell;
		var hmi = this.getCSHiddenMenuItems();
		var hidden = hmi.getHiddenMenuItems();
		var box = document.getElementById('hiddenMenuItems');

		while(box.firstChild) {
			box.removeChild(box.firstChild);
		}

		if (!hidden) return false;

		hidden = hidden.split(' ');
		for (var i=0; i<hidden.length; ++i) {
			if (!hidden[i]) continue;
			listitem = document.createElement('listitem');
			listitem.setAttribute('value',hidden[i]);
			listcell = document.createElement('listcell');
			listcell.className = 'listcell-iconic cs' + hidden[i].substr(hidden[i].indexOf('-'));
			listcell.setAttribute('label',hidden[i]);
			listitem.appendChild(listcell);
			box.appendChild(listitem);
		}
		return false;
	},

	onDialogAccept: function() {
		var dom = this.getDOM();
		var prefs = this.getPrefs();
		var gPrefs = this.getGlobalPrefs();

		//set cookiesafe checkbox prefs
		for (var i=0; i<dom.chk.cs.length; ++i) {
			prefs.setBoolPref(dom.chk.cs[i].id,dom.chk.cs[i].checked);
		}

		//set global checkbox prefs
		for (var i=0; i<dom.chk.gbl.length; ++i) {
			gPrefs.setBoolPref(dom.chk.gbl[i].id,dom.chk.gbl[i].checked);
		}

		//set cookiesafe menulist prefs
		for (var i=0; i<dom.mnu.cs.length; ++i) {
			if (dom.mnu.cs[i].id=='defaultSkin') {
				prefs.setCharPref(dom.mnu.cs[i].id,dom.mnu.cs[i].value);
			} else {
				prefs.setIntPref(dom.mnu.cs[i].id,dom.mnu.cs[i].value);
			}
		}

		//set global menulist prefs
		for (var i=0; i<dom.mnu.gbl.length; ++i) {
			gPrefs.setIntPref(dom.mnu.gbl[i].id,dom.mnu.gbl[i].value);
		}

		//set global textbox prefs
		gPrefs.setIntPref(dom.txt.gbl[0].id,dom.txt.gbl[0].value);
	},

	getDOM: function() {
		var checkboxes = this.isGlobal(document.getElementsByTagName('checkbox'));
		var menulists = this.isGlobal(document.getElementsByTagName('menulist'));
		var textboxes = this.isGlobal(document.getElementsByTagName('textbox'));

		return {chk:checkboxes,mnu:menulists,txt:textboxes};
	},

	isGlobal: function(items) {
		var global = [];
		var cookiesafe = [];
		for (var i=0; i<items.length; ++i) {
			if (items[i].getAttribute('global')) {
				global.push(items[i]);
			} else {
				cookiesafe.push(items[i]);
			}
		}
		return {gbl:global,cs:cookiesafe};
	},

	convertSkinUrl: function(url) {
		var index;
		switch (url) {
			case 'chrome://cookiesafe/skin/': index = 0;
                                                          break;

			case 'chrome://cookiesafe/content/iconsets/hand/': index = 1;
                                                                           break;

			case 'chrome://cookiesafe/content/iconsets/sweets/': index = 2;
                                                                             break;

			case 'chrome://cookiesafe/content/iconsets/simple/': index = 3;
                                                                             break;

			case 'chrome://cookiesafe/content/iconsets/simple3d/': index = 4;
                                                                             break;

			default: index = 0;
		}
		return index;
	},

	showMenuItem: function() {
		var hmi = this.getCSHiddenMenuItems();
		var box = document.getElementById('hiddenMenuItems');

		for (var i=0; i<box.selectedItems.length; ++i) {
			hmi.removeHiddenMenuItems(box.selectedItems[i].value);
		}

		var os = this.getObserver();
		os.notifyObservers(null,'cookiesafe-menuitems-changed','multiple');
	},

	isValid: function(box) {
		box.value = box.value.replace(/[a-zA-Z~`!@#$%^&*()_+|\-=\\{}\[\]:\";\'<>?,.\/ ]/g,'');
		if (box.value.length>1 && parseInt(box.value.charAt(0))==0) box.value = '';
	},

	lifetimeDays: function(tf) {
		document.getElementById('lifetimeDays').setAttribute('collapsed',tf);
	},

	viewAlert: function(str) {
		window.openDialog("chrome://cookiesafe/content/cookiesafeAlert.xul","_blank", "modal,centerscreen", str);
	}
};
