/***************************************************************************
Name: CS Lite
Description: Control cookie permissions.
Author: Ron Beckman
Homepage: http://forum.softwareblaze.com

Copyright (C) 2007  Ron Beckman

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

Free Software Foundation, Inc.
51 Franklin Street
Fifth Floor
Boston, MA  02110-1301
USA
***************************************************************************/




/*csInitPermObserver() is used to prevent an overflow of perm-changed
 notifications from being sent to the perm observer in rapid succession. that
 can occur by deleting multiple cookie exceptions, or by updating the blocklist
 which may add hundreds of exceptions.*/

function csInitPermObserver() {
	gPermissionManager.init();
}

var gPermissionManager = {

	lastCalled: 0,

	getStr: function(name) {
		return document.getElementById("cookiesafe-bundle").getString(name);
	},

	manageTreeBox: function(item) {
		if (item.id=='url') {
			var tree = document.getElementById('permissionsTree');
			tree.setAttribute('index',tree.currentIndex);
			if (tree.view && tree.view.selection) tree.view.selection.clearSelection();
			tree.currentIndex = -1;
		} else {
			var box = document.getElementById('url');
			box.value = '';
		}
	},

	getURI: function(url) {
		return Components.classes["@mozilla.org/network/io-service;1"].
		getService(Components.interfaces.nsIIOService).
		newURI(url,null,null);
	},

	getAppInfo: function() {
		return Components.classes['@mozilla.org/xre/app-info;1'].
		createInstance(Components.interfaces.nsIXULAppInfo);
	},

	getPermManager: function() {
		//check if browser is TB2
		var brows = this.getAppInfo();
		var num = parseInt(brows.version);
		if (brows.name=='Thunderbird' && num==2) {
			return Components.classes["@mozilla.org/CSPermManager;1"].
			getService(Components.interfaces.nsICSPermManager);
		} else {
			return Components.classes["@mozilla.org/permissionmanager;1"].
			getService(Components.interfaces.nsIPermissionManager);
		}
	},

	getObserver: function() {
		return Components.classes["@mozilla.org/observer-service;1"].
		getService(Components.interfaces.nsIObserverService);
	},

	init: function (aParams) {
		var params = window.arguments[0];

		//display description for cookie permissions
		var permissionsText = document.getElementById("permissionsText");
		while (permissionsText.hasChildNodes()) {
			permissionsText.removeChild(permissionsText.firstChild);
		}
		permissionsText.appendChild(document.createTextNode(params.introText));

		document.title = params.windowTitle;

		var os = this.getObserver();
		os.addObserver(this, "perm-changed", false);
		os.addObserver(this, "csperm-changed", false);

		//determine which column to sort by
		var tree = document.getElementById('permissionsTree');
		var sortBy = tree.getAttribute('lastClicked');
		if (!sortBy) {
			sortBy = 'host';
			tree.setAttribute('lastClicked',sortBy);
		}

		//determine whether to sort a to z
		var col = document.getElementById(sortBy);
		var atoz = col.getAttribute('atoz');
		atoz = (!atoz || atoz=='true') ? true : false;
		col.setAttribute('atoz',atoz);

		this.loadPermissions(sortBy,atoz);
	},

	uninit: function () {
		var os = this.getObserver();
		os.removeObserver(this, "perm-changed");
		os.removeObserver(this, "csperm-changed");
	},

	observe: function (aSubject, aTopic, aData) {
		if (aTopic != "perm-changed" && 
		    aTopic != "csperm-changed") return false;

		//determine which column to sort by
		var tree = document.getElementById('permissionsTree');
		var sortBy = tree.getAttribute('lastClicked');
		if (!sortBy) {
			sortBy = 'host';
			tree.setAttribute('lastClicked',sortBy);
		}

		//save the currently selected row
		var selection = (tree.currentIndex==-1) ? tree.getAttribute('index') : tree.currentIndex;
		selection = parseInt(selection);

		//determine whether to sort a to z
		var col = document.getElementById(sortBy);
		var atoz = col.getAttribute('atoz');
		atoz = (!atoz || atoz=='true') ? true : false;
		col.setAttribute('atoz',atoz);

		this.loadPermissions(sortBy,atoz);

		if (!isNaN(selection) && selection > -1) {
			tree.treeBoxObject.ensureRowIsVisible(selection);
		}

		/*check the number of milliseconds between notifications and uninit
		if necessary to prevent overflow and speed up process of adding or
		deleting multiple exceptions at once*/

		var dt = new Date();
		var time = dt.getTime();
		if ((time - this.lastCalled) < 1000) {
			this.uninit();
			window.setTimeout(csInitPermObserver,1050);
		}
		this.lastCalled = time;

		return false;
	},

	loadPermissions: function(sortBy,atoz) {
		var perms = this.getAllPerms(sortBy);
		if (!atoz) perms.reverse();

		var tree = document.getElementById('permissionsTree');
		//clear tree selection
		if (tree.view && tree.view.selection) {
			tree.view.selection.clearSelection();
		}

		//remove all tree items
		var child = document.getElementById('permissionsChild');
		while (child.firstChild) {
			child.removeChild(child.firstChild);
		}

		for (var i=0; i<perms.length; ++i) {
			child.appendChild(this.createItem(perms[i]));
		}
	},

	createItem: function(line) {
		var item = document.createElement('treeitem');
		var row = document.createElement('treerow');

		for (var i=1; i<line.length; ++i) {
			var cell = document.createElement('treecell');
			cell.setAttribute('label',line[i]);
			row.appendChild(cell);
		}

		item.appendChild(row);
		return item;
	},

	getCapString: function(num) {
		switch (parseInt(num)) {
			case 1: return this.getStr('cookiesafe.lAllow');
			case 2: return this.getStr('cookiesafe.lBlock');
			case 8: return this.getStr('cookiesafe.lSession');
			default: return '';
		}
	},

	getAllPerms: function(sortBy) {
		var exc = [];
		var mngr = this.getPermManager();
		if (mngr instanceof Components.interfaces.nsIPermissionManager) {
			var perms = mngr.enumerator;
			while (('hasMoreElements' in perms && perms.hasMoreElements()) ||
				 ('hasMore' in perms && perms.hasMore())) {
				var perm = perms.getNext();
				perm.QueryInterface(Components.interfaces.nsIPermission);
				var capability = this.getCapString(perm.capability);
				exc.push([(sortBy=='host') ? perm.host : capability,perm.host,capability]);
			}
		} else {
			var perms = mngr.getAllPermissions();
			if (!perms) return [];

			perms = perms.split(' ');
			for (var i=0; i<perms.length; ++i) {
				var temp = perms[i].split('|');
				var capability = this.getCapString(temp[1]);
				exc.push([(sortBy=='host') ? temp[0] : capability,temp[0],capability]);
			}
		}

		exc.sort();
		return exc;
	},

	addPermission: function(action) {
		var box = document.getElementById('url');
		var perm = box.value.replace(/\W*$/,'');
		if (!perm) {
			var tree = document.getElementById('permissionsTree');

			//make sure that a host is selected
			if (!tree.view || !tree.view.selection || tree.currentIndex==-1) return false;

			perm = tree.view.getCellText(tree.currentIndex,tree.columns.getColumnAt(0));
			var capability = tree.view.getCellText(tree.currentIndex,tree.columns.getColumnAt(1));

			//make sure that the capability is different
			if (this.getCapString(action)==capability) return false;
		}

		var scheme = document.getElementById('scheme');
		var uri = this.getURI((scheme.selectedIndex==0) ? 'http://'+perm : 'file:///mozilla');

		var mngr = this.getPermManager();
		mngr.add(uri,'cookie',action);
		box.value = '';

		var brows = this.getAppInfo();
		var num = parseInt(brows.version);
		if (brows.name=='Thunderbird' && num==2) {
			var os = this.getObserver();
			os.notifyObservers(null,'csperm-changed','added');
		}
		return false;
	},

	removePerm: function() {
		var box = document.getElementById('url');
		var perm = box.value.replace(/\W*$/g,'');
		if (!perm) {
			var tree = document.getElementById('permissionsTree');

			//make sure that a host is selected
			if (!tree.view || !tree.view.selection || tree.currentIndex==-1) return false;

			perm = tree.view.getCellText(tree.currentIndex,tree.columns.getColumnAt(0));
			var capability = tree.view.getCellText(tree.currentIndex,tree.columns.getColumnAt(1));
		}

		var mngr = this.getPermManager();
		mngr.remove(perm,'cookie');
		box.value = '';

		var brows = this.getAppInfo();
		var num = parseInt(brows.version);
		if (brows.name=='Thunderbird' && num==2) {
			var os = this.getObserver();
			os.notifyObservers(null,'csperm-changed','deleted');
		}
		return false;
	},

	removeAllPerms: function() {
		var mngr = this.getPermManager();
		mngr.removeAll();

		var brows = this.getAppInfo();
		var num = parseInt(brows.version);
		if (brows.name=='Thunderbird' && num==2) {
			var os = this.getObserver();
			os.notifyObservers(null,'csperm-changed','cleared');
		}
	},

	validate: function(box) {
		box.value = box.value.replace(/[^a-zA-Z0-9\-\.]|^\W*/g,'');
	},

	sortCol: function(sortBy) {
		//determine which column to sort by
		var tree = document.getElementById('permissionsTree');
		var lastClicked = tree.getAttribute('lastClicked');

		//determine whether to sort a to z
		var col = document.getElementById(sortBy);
		var atoz = col.getAttribute('atoz');
		if (sortBy==lastClicked) {
			atoz = (atoz && atoz=='true') ? false : true;
		} else {
			atoz = (!atoz || atoz=='true') ? true : false;
		}
		col.setAttribute('atoz',atoz);

		tree.setAttribute('lastClicked',sortBy);

		this.loadPermissions(sortBy,atoz);
	}
};